@@ -23,6 +23,7 @@
 #include <QFile>
 #include <QThreadPool>
 
+#include <KDebug>
 #include <KGlobalSettings>
 #include <KUrl>
 #include <KIO/NetAccess>
@@ -39,6 +40,7 @@ Picture::Picture(QObject *parent)
         : QObject(parent)
 {
     m_defaultImage = KGlobal::dirs()->findResource("data", "plasma-applet-frame/picture-frame-default.jpg");
+    m_checkDir = false;
 
     // listen for changes to the file we're displaying
     m_fileWatch = new KDirWatch(this);
@@ -82,7 +84,11 @@ void Picture::setPicture(const KUrl &currentUrl)
         connect(m_job, SIGNAL(finished(KJob*)), this, SLOT(slotFinished(KJob*)));
         emit pictureLoaded(defaultPicture(i18n("Loading image...")));
     } else {
-        if (currentUrl.isEmpty()) {
+        if (m_checkDir) {
+            loader = new ImageLoader(m_defaultImage);
+            m_message = i18nc("Info", "Dropped folder is empty. Please drop a folder with image(s)");
+            m_checkDir = false;
+        } else if (currentUrl.isEmpty()) {
             loader = new ImageLoader(m_defaultImage);
             m_message = i18nc("Info", "Put your photo here or drop a folder to start a slideshow");
             kDebug() << "default image ...";
@@ -121,6 +127,11 @@ void Picture::reload()
     QThreadPool::globalInstance()->start(loader);
 }
 
+void Picture::customizeEmptyMessage() 
+{
+    m_checkDir = true;
+}
+
 void Picture::slotFinished( KJob *job )
 {
     QString filename = m_currentUrl.fileName();
@@ -139,7 +150,7 @@ void Picture::slotFinished( KJob *job )
         setPath(path);
     }
     
-    emit checkImageLoaded(ImageLoader::correctRotation(image));
+    emit checkImageLoaded(ImageLoader::correctRotation(image, path));
 }
 
 void Picture::checkImageLoaded(QImage newImage)
