@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Thomas Georgiou <TAGeorgiou@gmail.com>          *
- *                         Artur Duque de Souza <morpheuz@gmail.com>       *
+ *                         Artur Duque de Souza <asouza@kde.org>           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -28,7 +28,8 @@
 
 #include <kmimetype.h>
 #include <KTemporaryFile>
-
+#include <KDE/KIO/MimetypeJob>
+#include <KDebug>
 
 #include <kio/global.h>
 #include <kio/job.h>
@@ -36,6 +37,7 @@
 #include <Plasma/Theme>
 #include <Plasma/Service>
 #include <Plasma/ServiceJob>
+#include <KNS3/DownloadDialog>
 
 #include <QApplication>
 #include <QClipboard>
@@ -52,9 +54,9 @@
 
 Pastebin::Pastebin(QObject *parent, const QVariantList &args)
     : Plasma::Applet(parent, args),
-      m_textBackend(0), m_imageBackend(0), m_imagePrivacy(0),
-      m_historySize(3), m_signalMapper(new QSignalMapper()), m_paste(0),
-      m_topSeparator(0), m_bottomSeparator(0), m_servers(0)
+      m_signalMapper(new QSignalMapper()), m_paste(0),
+      m_topSeparator(0), m_bottomSeparator(0),
+      m_historySize(3), m_newStuffDialog(0)
 {
     setAcceptDrops(true);
     setHasConfigurationInterface(true);
@@ -62,29 +64,35 @@ Pastebin::Pastebin(QObject *parent, const QVariantList &args)
     setMinimumSize(16, 16);
     resize(150, 150);
     m_timer = new QTimer(this);
-    m_hoverText = i18n("Drop!");
     connect(m_timer, SIGNAL(timeout()), this, SLOT(showErrors()));
 
     connect(m_signalMapper, SIGNAL(mapped(const QString &)),
              this, SLOT(copyToClipboard(const QString &)));
     connect(this, SIGNAL(activate()), this, SLOT(postClipboard()));
 
-    Plasma::DataEngine *engine = dataEngine("pastebin");
-    m_postingService = engine->serviceForSource("");
-    connect(m_postingService, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(postingFinished(Plasma::ServiceJob*)));
-    engine->connectSource("textservers", this);
-    engine->connectSource("imageservers", this);
+    // connect to all sources of our 'share' dataengine
+    m_engine = dataEngine("org.kde.plasma.dataengine.share");
+    m_engine->connectAllSources(this);
+
+    // to detect when the mimetypes were added again after a refresh
+    connect(m_engine, SIGNAL(sourceAdded(const QString&)),
+            this, SLOT(sourceAdded(const QString&)));
+    connect(m_engine, SIGNAL(sourceRemoved(const QString&)),
+            this, SLOT(sourceRemoved(const QString&)));
 }
 
 Pastebin::~Pastebin()
 {
     delete m_topSeparator;
     delete m_bottomSeparator;
-    delete m_postingService;
+    delete m_newStuffDialog;
+
+    // save history of URLs
     QString history;
-    for (int i = 0; i < m_actionHistory.size(); ++i) {
+    const int numberOfActionHistory = m_actionHistory.size();
+    for (int i = 0; i < numberOfActionHistory; ++i) {
         history.prepend(m_actionHistory.at(i)->toolTip());
-        history.prepend("|");
+        history.prepend('|');
         delete m_actionHistory.at(i);
     }
 
@@ -94,20 +102,7 @@ Pastebin::~Pastebin()
 
 void Pastebin::init()
 {
-    KConfigGroup cg = config();
-
-    m_textBackend = cg.readEntry("TextBackend", "0").toInt();
-    m_imageBackend = cg.readEntry("ImageBackend", "0").toInt();
-    m_imagePrivacy = cg.readEntry("imagePrivacy", "0").toInt();
-    int historySize = cg.readEntry("HistorySize", "3").toInt();
-
-    QStringList history = cg.readEntry("History", "").split('|', QString::SkipEmptyParts);
-
-    for (int i = 0; i < history.size(); ++i) {
-        addToHistory(history.at(i));
-    }
-
-    setHistorySize(historySize);
+    configChanged();
     setActionState(Idle);
     setInteractionState(Waiting);
 
@@ -117,9 +112,49 @@ void Pastebin::init()
     Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 }
 
-void Pastebin::dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
+void Pastebin::dataUpdated(const QString &source, const Plasma::DataEngine::Data &data)
 {
-    //TODO: use the mappings!
+    // update the options
+    if (source != "Mimetypes") {
+        const QString mimetype = data.value("Mimetypes").toString();
+
+        if (mimetype.startsWith("text/")) {
+            m_txtServers.insert(data.value("Name").toString(), source);
+        } else if (mimetype.startsWith("image/")) {
+            m_imgServers.insert(data.value("Name").toString(), source);
+        } else {
+            kDebug() << "Mimetype not supported by this applet";
+        }
+    }
+}
+
+void Pastebin::sourceAdded(const QString &source)
+{
+    // update the options
+    if (source != "Mimetypes") {
+        const Plasma::DataEngine::Data data = m_engine->query(source);
+        const QString mimetype = data.value("Mimetypes").toString();
+
+        if (mimetype.startsWith("text/")) {
+            m_txtServers.insert(data.value("Name").toString(), source);
+        } else if (mimetype.startsWith("image/")) {
+            m_imgServers.insert(data.value("Name").toString(), source);
+        } else {
+            kDebug() << "Mimetype not supported by this applet";
+        }
+    }
+}
+
+void Pastebin::sourceRemoved(const QString &source)
+{
+    // update the options
+    if (source != "Mimetypes") {
+        QString key = m_txtServers.key(source);
+        m_txtServers.remove(key);
+
+        key = m_imgServers.key(source);
+        m_imgServers.remove(key);
+    }
 }
 
 void Pastebin::setHistorySize(int max)
@@ -220,11 +255,6 @@ void Pastebin::setActionState(ActionState state)
 
 void Pastebin::constraintsEvent(Plasma::Constraints constraints)
 {
-    if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
-        QFontMetrics fm(m_font);
-        int textWidthInPixels = fm.width(m_hoverText);
-        m_font.setPointSize((int)(m_font.pointSize() * ((double)contentsRect().width() / textWidthInPixels)));
-    }
     if (constraints & Plasma::FormFactorConstraint) {
         if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical ) {
             setPreferredSize(-1, -1);
@@ -340,31 +370,6 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
     qreal proportion = contentsRect.width() / contentsRect.height();
     qreal round_radius = 35.0;
     p->drawRoundedRect(contentsRect, round_radius / proportion, round_radius, Qt::RelativeSize);
-
-    if (m_interactionState == DraggedOver) {
-        m_fgColor.setAlphaF(m_alpha * 0.8);
-    } else if (m_interactionState == Hovered) {
-        m_fgColor.setAlphaF(m_alpha * 0.6);
-    }
-
-    p->drawText(contentsRect, Qt::AlignCenter, m_hoverText);
-    p->setPen(m_fgColor);
-
-    if (m_actionState == IdleSuccess) {
-        QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
-        p->setFont(font);
-        p->drawText(contentsRect, Qt::AlignCenter, m_url);
-    } else if (m_actionState == IdleError) {
-        QFont font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
-        font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
-        font.setBold(true);
-        p->setFont(font);
-        p->drawText(contentsRect, Qt::AlignCenter, i18n("Error: Try Again"));
-    } else {
-        p->setFont(m_font);
-        p->drawText(contentsRect, Qt::AlignCenter, m_hoverText);
-    }
 }
 
 void Pastebin::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
@@ -418,6 +423,40 @@ void Pastebin::animationUpdate(qreal progress)
     update();
 }
 
+void Pastebin::getNewStuff()
+{
+    if (!m_newStuffDialog) {
+        QString ghns("pastebin.knsrc");
+        m_newStuffDialog = new KNS3::DownloadDialog( ghns );
+        connect(m_newStuffDialog, SIGNAL(accepted()),
+                this, SLOT(newStuffFinished()));
+    }
+    m_newStuffDialog->show();
+}
+
+void Pastebin::newStuffFinished()
+{
+    if ( m_newStuffDialog->changedEntries().count() ) {
+        // refresh the options of config dialog
+        refreshConfigDialog();
+
+        // setup the config dialog to last options
+        KConfigGroup cg = config();
+        uiConfig.textServer->setCurrentItem(cg.readEntry("TextProvider", ""));
+        uiConfig.imageServer->setCurrentItem(cg.readEntry("ImageProvider", ""));
+    }
+}
+
+void Pastebin::refreshConfigDialog()
+{
+    // setup text
+    uiConfig.textServer->clear();
+    uiConfig.textServer->addItems(m_txtServers.keys());
+
+    // setup image
+    uiConfig.imageServer->clear();
+    uiConfig.imageServer->addItems(m_imgServers.keys());
+}
 
 void Pastebin::createConfigurationInterface(KConfigDialog *parent)
 {
@@ -427,158 +466,50 @@ void Pastebin::createConfigurationInterface(KConfigDialog *parent)
     uiConfig.setupUi(general);
 
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
-    connect(parent, SIGNAL(cancelClicked()), this, SLOT(closeServerDialog()));
     parent->addPage(general, i18n("General"), Applet::icon());
 
-    uiConfig.textServer->setCurrentIndex(m_textBackend);
-    uiConfig.imageServer->setCurrentIndex(m_imageBackend);
+    uiConfig.ghnsButton->setIcon(KIcon("get-hot-new-stuff"));
+    connect(uiConfig.ghnsButton, SIGNAL(clicked()), this, SLOT(getNewStuff()));
+
+    refreshConfigDialog();
+    uiConfig.textServer->setCurrentItem(cg.readEntry("TextProvider", m_txtServers.keys().at(0)));
+    uiConfig.imageServer->setCurrentItem(cg.readEntry("ImageProvider", m_imgServers.keys().at(0)));
     uiConfig.historySize->setValue(m_historySize);
-    uiConfig.imagePrivacy->setCurrentIndex(m_imagePrivacy);
-
-    uiConfig.editTextServer->setIcon(KIcon("document-edit"));
-    uiConfig.editImageServer->setIcon(KIcon("document-edit"));
-
-    connect(uiConfig.editTextServer, SIGNAL(clicked()), this, SLOT(editTextServer()));
-    connect(uiConfig.editImageServer, SIGNAL(clicked()), this, SLOT(editImageServer()));
-}
-
-void Pastebin::editTextServer()
-{
-    KConfigGroup cg = config();
-
-    // avoid memory leaks as its not modal and the user can click again
-    // on the edit button
-    delete m_servers;
-
-    m_servers = new QWidget();
-    uiServers.setupUi(m_servers);
-    m_servers->show();
-    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveTextServer()));
-    connect(uiServers.buttonBox, SIGNAL(rejected()), this, SLOT(closeServerDialog()));
-    QString serverAddress;
-    switch(uiConfig.textServer->currentIndex()) {
-        case Pastebin::PASTEBINCA:
-            serverAddress = cg.readEntry("pastebinca", "http://pastebin.ca");
-            break;
-
-        case Pastebin::PASTEBINCOM:
-           serverAddress = cg.readEntry("pastebincom", "http://pastebin.com");
-           break;
-    }
-
-    uiServers.server->setText(serverAddress);
-}
-
-void Pastebin::saveTextServer()
-{
-    KConfigGroup cg = config();
-
-    switch(uiConfig.textServer->currentIndex()) {
-        case Pastebin::PASTEBINCA:
-            cg.writeEntry("pastebinca", uiServers.server->text());
-            break;
-
-        case Pastebin::PASTEBINCOM:
-            cg.writeEntry("pastebincom", uiServers.server->text());
-            break;
-    }
-
-    closeServerDialog();
-}
-
-void Pastebin::closeServerDialog()
-{
-    delete m_servers;
-    m_servers = 0;
-}
-
-void Pastebin::editImageServer()
-{
-    KConfigGroup cg = config();
-
-    // avoid memory leaks as its not modal and the user can click again
-    // on the edit button
-    delete m_servers;
-
-    m_servers = new QWidget();
-    uiServers.setupUi(m_servers);
-    m_servers->show();
-    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveImageServer()));
-    connect( uiServers.buttonBox, SIGNAL( rejected() ), this, SLOT( closeServerDialog() ) );
-    QString serverAddress;
-    switch(uiConfig.imageServer->currentIndex()) {
-        case Pastebin::IMAGEBINCA:
-            serverAddress = cg.readEntry("imagebinca", "http://imagebin.ca");
-            break;
-
-        case Pastebin::IMAGESHACK:
-            serverAddress = cg.readEntry("imageshack", "http://imageshack.us");
-            break;
-
-        case Pastebin::SIMPLESTIMAGEHOSTING:
-            serverAddress = cg.readEntry("simplestimagehosting",
-                                         "http://api.simplest-image-hosting.net");
-            break;
-
-        case Pastebin::IMGUR:
-            serverAddress = cg.readEntry("imgur", "http://imgur.com/api/upload");
-            break;
-    }
-
-    uiServers.server->setText(serverAddress);
-}
-
-void Pastebin::saveImageServer()
-{
-    KConfigGroup cg = config();
-
-    switch(uiConfig.imageServer->currentIndex()) {
-        case Pastebin::IMAGEBINCA:
-            cg.writeEntry("imagebinca", uiServers.server->text());
-            break;
-
-        case Pastebin::IMAGESHACK:
-            cg.writeEntry("imageshack", uiServers.server->text());
-            break;
-
-        case Pastebin::SIMPLESTIMAGEHOSTING:
-            cg.writeEntry("simplestimagehosting", uiServers.server->text());
-            break;
-
-        case Pastebin::IMGUR:
-            cg.writeEntry("imgur", uiServers.server->text());
-            break;
-
-    }
-    closeServerDialog();
 }
 
 void Pastebin::configAccepted()
 {
     KConfigGroup cg = config();
-    m_textBackend = uiConfig.textServer->currentIndex();
-    m_imageBackend = uiConfig.imageServer->currentIndex();
-    m_imagePrivacy = uiConfig.imagePrivacy->currentIndex();
-
     int historySize = uiConfig.historySize->value();
     setHistorySize(historySize);
 
-    cg.writeEntry("TextBackend", m_textBackend);
-    cg.writeEntry("ImageBackend", m_imageBackend);
-    cg.writeEntry("imagePrivacy", m_imagePrivacy);
+    cg.writeEntry("TextProvider", uiConfig.textServer->currentText());
+    cg.writeEntry("ImageProvider", uiConfig.imageServer->currentText());
     cg.writeEntry("HistorySize", historySize);
 
-    closeServerDialog();
     emit configNeedsSaving();
 }
 
+void Pastebin::configChanged()
+{
+    KConfigGroup cg = config();
+    int historySize = cg.readEntry("HistorySize", "3").toInt();
+    QStringList history = cg.readEntry("History", "").split('|', QString::SkipEmptyParts);
+
+    m_actionHistory.clear();
+    const int numberOfItems = history.size();
+    for (int i = 0; i < numberOfItems; ++i) {
+        addToHistory(history.at(i));
+    }
+    setHistorySize(historySize);
+}
+
 void Pastebin::showResults(const QString &url)
 {
     m_timer->stop();
     m_url = url;
     setActionState(IdleSuccess);
     copyToClipboard(url);
-
     addToHistory(url);
 }
 
@@ -592,7 +523,7 @@ void Pastebin::copyToClipboard(const QString &url)
     KNotification *notify = new KNotification("urlcopied");
     notify->setComponentData(KComponentData("plasma_pastebin"));
     notify->setText(i18nc("Notification when the pastebin applet has copied the URL to the clipboard",
-                          "%1 has been copied to your clipboard", url));
+                          "The URL for your paste has been copied to the clipboard"));
     notify->setPixmap(pix);
     notify->setActions(QStringList(i18n("Open browser")));
     connect(notify, SIGNAL(action1Activated()), this, SLOT(openLink()));
@@ -772,27 +703,39 @@ void Pastebin::postClipboard(bool preferSelection)
 #endif //Q_WS_WIN
 }
 
-void Pastebin::postContent(QString text, QImage imageData)
+void Pastebin::postContent(QString text, const QImage& imageData)
 {
+    QString sourceName;
     KUrl testPath(text);
-    bool validPath = QFile::exists(testPath.toLocalFile());
-    bool image = false;
-    QString operationName;
+    bool validPath = true;
+
+    // use KIO to check if the file exists using mimetype job
+    KIO::MimetypeJob *mjob = KIO::mimetype(testPath);
+    if (!mjob->exec()) {
+        // it's not a file - usually this happens when we are
+        // just sharing plain text or image
+        validPath = false;
+    }
+
+    KConfigGroup cg = config();
+    // This is needed to provide smooth transition between old config and new one
+    const QString txtProvider = cg.readEntry("TextProvider", m_txtServers.keys().at(0));
+    const QString imgProvider = cg.readEntry("ImageProvider", m_imgServers.keys().at(0));
 
     if (validPath) {
         KMimeType::Ptr type = KMimeType::findByPath(testPath.path());
 
         if (type->name().indexOf("image/") != -1) {
-            operationName = "image";
-            image = true;
+            // its image
+            sourceName = m_imgServers.value(imgProvider);
         } else {
-            operationName = "text";
+            // its text
+            sourceName = m_txtServers.value(txtProvider);
         }
     } else if (imageData.isNull()) {
-        operationName = "literalText";
+        sourceName = m_txtServers.value(txtProvider);
     } else {
-        operationName = "image";
-        image = true;
+        sourceName = m_imgServers.value(imgProvider);
 
         KTemporaryFile tempFile;
         if (tempFile.open()) {
@@ -814,64 +757,33 @@ void Pastebin::postContent(QString text, QImage imageData)
         }
     }
 
-    KConfigGroup cg = config();
+    kDebug() << "Is valid path: " << validPath;
+    kDebug() << "Provider used: " << sourceName;
 
-    KConfigGroup ops = m_postingService->operationDescription(operationName);
-    kDebug() << image << validPath << operationName;
-    if (image) {
-        switch(m_imageBackend) {
-            case Pastebin::IMAGEBINCA:
-                ops.writeEntry("server", cg.readEntry("imagebinca", "http://imagebin.ca"));
-                break;
-
-            case Pastebin::IMAGESHACK:
-                ops.writeEntry("server", cg.readEntry("imageshack", "http://imageshack.us"));
-                break;
-
-            case Pastebin::SIMPLESTIMAGEHOSTING:
-                ops.writeEntry("server", cg.readEntry("simplestimagehosting",
-                                                      "http://api.simplest-image-hosting.net"));
-                break;
-
-            case Pastebin::IMGUR:
-                ops.writeEntry("server", cg.readEntry("imgur", "http://imgur.com/api/upload"));
-                break;
-        }
-
-        ops.writeEntry("fileName", text);
-        ops.writeEntry("privacy", m_imagePrivacy);
-        ops.writeEntry("backend", m_imageBackend);
-    } else {
-        switch (m_textBackend) {
-            case Pastebin::PASTEBINCA:
-                ops.writeEntry("server", cg.readEntry("pastebinca", "http://pastebin.ca"));
-                break;
-
-            case Pastebin::PASTEBINCOM:
-                ops.writeEntry("server", cg.readEntry("pastebincom", "http://pastebin.com"));
-                break;
-        }
-
-        if (validPath) {
-            ops.writeEntry("fileName", text);
-        } else {
-            ops.writeEntry("text", text);
-        }
-
-        ops.writeEntry("backend", m_textBackend);
+    if (sourceName.isEmpty()) {
+        // no provider was configured
+        showErrors();
+        return;
     }
 
+    m_postingService = m_engine->serviceForSource(sourceName);
+    KConfigGroup ops = m_postingService->operationDescription("share");
+    ops.writeEntry("content", text);
+
     Plasma::ServiceJob *job = m_postingService->startOperationCall(ops);
+    connect(job, SIGNAL(finished(KJob*)), this, SLOT(postingFinished(KJob*)));
+
     setActionState(Sending);
     m_timer->start(20000);
 }
 
-void Pastebin::postingFinished(Plasma::ServiceJob *job)
+void Pastebin::postingFinished(KJob *job)
 {
-    if (job->error()) {
+    Plasma::ServiceJob *sjob = static_cast<Plasma::ServiceJob*>(job);
+    if (sjob->error()) {
         showErrors();
     } else {
-        showResults(job->result().toString());
+        showResults(sjob->result().toString());
     }
 }
 
