@@ -23,7 +23,6 @@
 #include <Plasma/Containment>
 #include <Plasma/Theme>
 #include <Plasma/ToolTipManager>
-#include <plasma/weather/weatherutils.h>
 #include <KUnitConversion/Converter>
 #include <plasmaweather/weatherconfig.h>
 #include <math.h>
@@ -59,7 +58,7 @@ void WeatherStation::init()
     if (hasAuthorization("LaunchApp")) {
         connect(m_lcd, SIGNAL(clicked(const QString&)), this, SLOT(clicked(const QString&)));
     }
-    
+
     m_lcdPanel = new LCD(this);
     m_lcdPanel->setSvg("weatherstation/lcd_panel");
     m_lcdPanel->setLabel("temperature-label", i18n("OUTDOOR TEMP"));
@@ -267,6 +266,7 @@ void WeatherStation::setTemperature(const Value& temperature, bool hasDigit)
 {
     hasDigit = hasDigit || (temperatureUnit() != temperature.unit());
     Value v = temperature.convertTo(temperatureUnit());
+    qDebug() << v.isValid();
     m_lcd->setLabel("temperature-unit-label", v.unit()->symbol());
     m_lcdPanel->setLabel("temperature-unit-label", v.unit()->symbol());
     QString tmp = hasDigit ? fitValue(v , 4) : QString::number(v.number());
