@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2009 by Sebastian K?gler <sebas@kde.org>                *
+ *   Copyright 2009-2011 by Sebastian Kugler <sebas@kde.org>               *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -27,11 +27,14 @@
 #include <QLabel>
 
 #include <QGraphicsLinearLayout>
+#include <QWebElement>
+#include <QWebFrame>
 
 // KDE
 #include <KDebug>
 #include <KConfigDialog>
 #include <KServiceTypeTrader>
+#include <Plasma/Theme>
 
 // Plasma
 #include <Plasma/Label>
@@ -39,8 +42,7 @@
 WebSlice::WebSlice(QObject *parent, const QVariantList &args)
     : Plasma::PopupApplet(parent, args),
     m_slice(0),
-    m_url(0),
-    m_size(192, 192)
+    m_url(0)
 {
     setPopupIcon("internet-web-browser");
     setAspectRatioMode(Plasma::IgnoreAspectRatio );
@@ -48,7 +50,8 @@ WebSlice::WebSlice(QObject *parent, const QVariantList &args)
     setAcceptsHoverEvents(true);
 
     setMinimumSize(64, 64);
-    resize(300, 300);
+    resize(800, 600);
+    kDebug() << "0";
 }
 
 void WebSlice::init()
@@ -69,8 +72,7 @@ void WebSlice::init()
             }
         }
     }
-
-    configChanged();
+    void graphicsWidget();
 }
 
 WebSlice::~WebSlice ()
@@ -79,6 +81,7 @@ WebSlice::~WebSlice ()
 
 void WebSlice::configChanged()
 {
+    kDebug();
     KConfigGroup cg = config();
     if (!m_url.isValid() || m_url.isEmpty()) {
         m_url = cg.readEntry("url", "http://dot.kde.org/");
@@ -89,10 +92,11 @@ void WebSlice::configChanged()
         //m_sliceGeometry = cg.readEntry("size", QRectF(258, 102, 550, 511));
         m_sliceGeometry = cg.readEntry("sliceGeometry", QRectF());
     }
-    m_size = cg.readEntry("size", m_size);
     setAssociatedApplicationUrls(KUrl::List(m_url));
-
-    //kDebug() << "url/element/slicegeometry:" << m_url << m_element << m_sliceGeometry;    
+    if (m_slice) {
+        m_slice->preview();
+    }
+    loadSlice(m_url, m_element);
 }
 
 QGraphicsWidget* WebSlice::graphicsWidget()
@@ -104,22 +108,27 @@ QGraphicsWidget* WebSlice::graphicsWidget()
 
 
         m_slice = new KGraphicsWebSlice(m_widget);
-        connect(m_slice, SIGNAL(sizeChanged(QSizeF)), this, SLOT(sizeChanged(QSizeF)));
+        m_slice->setMaximumSize(contentsRect().size());
+        //m_slice->show();
+        updateColors();
+        //connect(m_slice, SIGNAL(sizeChanged(QSizeF)), this, SLOT(sizeChanged(QSizeF)));
         connect(m_slice, SIGNAL(loadFinished(bool)), this, SLOT(loadFinished(bool)));
+        connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(updateColors()));
+
         setBusy(true);
         m_slice->setLoadingText(i18nc("displayed in the widget while loading", "<h1>Loading...</h1>"));
-        m_slice->setUrl(m_url);
-        m_slice->setElement(m_element);
-        m_slice->setSliceGeometry(m_sliceGeometry);
-        m_slice->hide();
+        //m_slice->hide();
         l->addItem(m_slice);
+        kDebug() << "slice set up";
+        configChanged();
     }
     return m_widget;
 }
 
 void WebSlice::createConfigurationInterface(KConfigDialog *parent)
 {
-    QLabel *info = new QLabel(parent);
+    //QLabel *info = new QLabel(parent);
+    QLabel* info = new QLabel(parent);
     info->setWordWrap(true);
     info->setTextInteractionFlags(Qt::TextBrowserInteraction);
     info->setText(i18n("<p>The Webslice Widget allows you to display a part of a webpage on your desktop or in a panel. The webslice is fully interactive.</p>"
@@ -136,21 +145,81 @@ void WebSlice::createConfigurationInterface(KConfigDialog *parent)
     QWidget *widget = new QWidget(parent);
     ui.setupUi(widget);
     parent->addPage(widget, i18nc("general config page", "Webpage"), Applet::icon());
+    connect(ui.loadUrl, SIGNAL(clicked()), this, SLOT(loadUrl()));
+    connect(ui.elementCombo, SIGNAL(destroyed()), SLOT(disconnectLoadFinished()));
+    connect(ui.elementCombo, SIGNAL(activated(const QString&)), this, SLOT(preview(const QString&)));
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
     ui.urlEdit->setText(m_url.toString());
+    ui.loadUrl->setIcon(KIcon("view-refresh"));
     ui.geometryEdit->setText(sliceGeometryToString());
-    ui.elementEdit->setText(m_element);
+    ui.elementCombo->setEditable(true);
+    updateElements();
+    preview(m_element);
+}
+
+void WebSlice::updateElements()
+{
+    ui.elementCombo->clear();
+    ui.elementCombo->addItem(m_element, m_element);
+    ui.elementCombo->addItem(QString("body"), QString("body"));
+    foreach(const QWebElement el, m_slice->page()->mainFrame()->findAllElements("*")) {
+        QString elSelector;
+        QString elAttributeName;
+        if (el.attributeNames().contains("id")) {
+            elAttributeName = QString("id");
+            elSelector = QString("#%1").arg(el.attribute("id")); // according to CSS selector syntax
+        } else {
+            /* don't add all elements for now, this really only clutters the combo
+            elSelector = el.localName();
+            if (!el.attribute("class").isEmpty()) {
+                elSelector.append("." + el.attribute("class"));
+            }
+            */
+        }
+        // Add Item?
+        if (!elSelector.isEmpty() && !(el.geometry().size().isNull())) {
+            ui.elementCombo->addItem(elSelector, elAttributeName);
+        }
+    }
+}
+
+void WebSlice::disconnectLoadFinished()
+{
+    // we need to prevent the combo from being updated when it's gone
+    disconnect(m_slice, SIGNAL(loadFinished(bool)), this, SLOT(updateElements()));
+    m_slice->preview();
+}
+
+void WebSlice::preview(const QString &selector)
+{
+    ui.geometryEdit->setText(sliceGeometryToString(selector));
+
+    m_slice->preview(selector);
+}
+
+void WebSlice::loadUrl()
+{
+    loadSlice(QUrl(ui.urlEdit->text()));
+    connect(m_slice, SIGNAL(loadFinished(bool)), SLOT(updateElements()));
+}
+
+void WebSlice::loadSlice(const QUrl &url, const QString &selector)
+{
+    if (m_slice) {
+        m_slice->loadSlice(url, selector);
+        setAssociatedApplicationUrls(KUrl::List(url));
+    }
 }
 
 void WebSlice::configAccepted()
 {
     if (m_url.toString() != ui.urlEdit->text() ||
-        m_element != ui.elementEdit->text() ||
+        m_element != ui.elementCombo->currentText() ||
         ui.geometryEdit->text() != sliceGeometryToString()) {
 
         m_url = QUrl(ui.urlEdit->text());
-        m_element = ui.elementEdit->text();
+        m_element = ui.elementCombo->currentText();
 
         QString geo = ui.geometryEdit->text();
         QStringList gel = geo.split(',');
@@ -166,7 +235,6 @@ void WebSlice::configAccepted()
             } else {
                 m_sliceGeometry = QRectF(x, y, w, h);
                 config().writeEntry("sliceGeometry", m_sliceGeometry);
-                //kDebug() << "new slice geometry:" << m_sliceGeometry;
             }
         } else {
             if (m_element.isEmpty()) {
@@ -174,11 +242,6 @@ void WebSlice::configAccepted()
             }
         }
 
-        m_slice->setUrl(m_url);
-        setAssociatedApplicationUrls(KUrl::List(m_url));
-        m_slice->setElement( m_element );
-        m_slice->hide();
-
         KConfigGroup cg = config();
         cg.writeEntry("url", m_url.toString());
         cg.writeEntry("element", m_element);
@@ -186,66 +249,29 @@ void WebSlice::configAccepted()
             m_sliceGeometry = QRectF();
         }
         emit configNeedsSaving();
-        //kDebug() << "config changed" << m_element << m_url;
+        configChanged();
     }
 }
 
-QString WebSlice::sliceGeometryToString()
+QString WebSlice::sliceGeometryToString(const QString &selector)
 {
-    QString s = QString("%1,%2,%3,%4").arg(m_sliceGeometry.x()).arg(m_sliceGeometry.y()).arg(m_sliceGeometry.width()).arg(m_sliceGeometry.height());
+    QString s = QString("%1,%2,%3,%4").arg(m_slice->sliceGeometry(selector).x()).arg(m_slice->sliceGeometry(selector).y()).arg(m_slice->sliceGeometry(selector).width()).arg(m_slice->sliceGeometry(selector).height());
     return s;
 }
 
-
-void WebSlice::constraintsEvent(Plasma::Constraints constraints)
-{
-    if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
-        //kDebug() << "Constraint changed:" << mapToScene(contentsRect());
-        if (m_slice) {
-            //kDebug() << "resizing slice to:" << contentsRect().size();
-            m_slice->setMaximumSize(contentsRect().size());
-            m_widget->setMinimumSize(64, 64);
-            m_slice->refresh();
-        }
-    }
-}
-
 void WebSlice::loadFinished(bool ok)
 {
     setBusy(false);
 
     if (!ok) {
-      // TODO set error image
-      kDebug() << "loading failed, but at least we know...";
       return;
     }
-
-    m_slice->show();
-    m_size = m_slice->geometry().size();
-
-    setAspectRatioMode(Plasma::KeepAspectRatio );
-
-    kDebug() << "done loading, resizing slice to:" << contentsRect().size();
-    m_slice->setMaximumSize(contentsRect().size());
 }
 
-void WebSlice::sizeChanged(QSizeF newsize)
+void WebSlice::updateColors()
 {
-    if (m_slice && m_size != newsize) {
-        QSizeF m_size = QSizeF(newsize.width() + 28, newsize.height() + 28);
-        m_slice->resize(m_size);
-
-        QRectF g = QRectF(mapToScene(contentsRect().topLeft()), m_size);
-        mapFromScene(contentsRect().topLeft());
-
-        m_widget->setMinimumSize(m_size);
-        m_slice->setMaximumSize(contentsRect().size());
-        setPreferredSize(m_size);
-        kDebug() << "size is now:" << m_size;
-        KConfigGroup cg = config();
-        cg.writeEntry("size", m_size);
-        emit configNeedsSaving();
-    }
+    m_slice->setPreviewMaskColor(
+        Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
 }
 
 #include "webslice.moc"
