@@ -18,16 +18,21 @@
  */
 
 #include "weatherconfig.h"
-#include "weatherconfigsearch.h"
+
+#include <QLineEdit>
+
+#include <KDebug>
+#include <KInputDialog>
+#include <KPixmapSequence>
+#include <kpixmapsequencewidget.h>
+//#include <KPixmapSequenceWidget>
+
+#include <KNS3/DownloadDialog>
+#include <KUnitConversion/Converter>
+
 #include "weathervalidator.h"
 #include "weatheri18ncatalog.h"
 #include "ui_weatherconfig.h"
-#include <KMessageBox>
-#include <KInputDialog>
-
-#include <KNS/Engine>
-#include <KUnitConversion/Converter>
-
 using namespace KUnitConversion;
 
 class WeatherConfig::Private
@@ -35,23 +40,45 @@ class WeatherConfig::Private
 public:
     Private(WeatherConfig *weatherconfig)
         : q(weatherconfig),
-          searchDlg(weatherconfig)
+          dataengine(0),
+          dlg(0),
+          busyWidget(0),
+          checkedInCount(0)
     {
     }
 
+    void setSource(int index)
+    {
+        QString text = ui.locationCombo->itemData(index).toString();
+        if (text.isEmpty()) {
+            ui.locationCombo->lineEdit()->setText(QString());
+        } else {
+            source = text;
+            emit q->settingsChanged();
+        }
+    }
+
     void changePressed()
     {
-        searchDlg.setSource(source);
-        searchDlg.show();
-    }
+        checkedInCount = 0;
+        QString text = ui.locationCombo->currentText();
 
-    void searchResult(int result)
-    {
-        if (result == QDialog::Accepted) {
-            q->setSource(searchDlg.source());
-            searchDlg.hide();
-        } else {
-            searchDlg.hide();
+        if (text.isEmpty()) {
+            return;
+        }
+
+        ui.locationCombo->clear();
+        ui.locationCombo->lineEdit()->setText(text);
+
+        if (!busyWidget) {
+            busyWidget = new KPixmapSequenceWidget(q);
+            KPixmapSequence seq(QLatin1String( "process-working" ), 22);
+            busyWidget->setSequence(seq);
+            ui.locationSearchLayout->insertWidget(1, busyWidget);
+        }
+
+        foreach (WeatherValidator *validator, validators) {
+            validator->validate(text, true);
         }
     }
 
@@ -62,12 +89,18 @@ public:
         }
     }
 
+    void addSources(const QMap<QString, QString> &sources);
+    void validatorError(const QString &error);
+
     WeatherConfig *q;
-    WeatherConfigSearch searchDlg;
-    Plasma::DataEngine *m_engine;
+    //QHash<QString, QString> ions;
+    QList<WeatherValidator *> validators;
+    Plasma::DataEngine *dataengine;
     QString source;
     Ui::WeatherConfig ui;
     KDialog *dlg;
+    KPixmapSequenceWidget *busyWidget;
+    int checkedInCount;
 };
 
 WeatherConfig::WeatherConfig(QWidget *parent)
@@ -101,7 +134,6 @@ WeatherConfig::WeatherConfig(QWidget *parent)
     d->ui.ghnsProviderButton->hide();
 
     connect(d->ui.changeButton, SIGNAL(clicked()), this, SLOT(changePressed()));
-    connect(&d->searchDlg, SIGNAL(finished(int)), this, SLOT(searchResult(int)));
 
     connect(d->ui.updateIntervalSpinBox, SIGNAL(valueChanged(int)),
             this, SLOT(setUpdateInterval(int)));
@@ -116,7 +148,8 @@ WeatherConfig::WeatherConfig(QWidget *parent)
             this, SIGNAL(settingsChanged()));
     connect(d->ui.visibilityComboBox, SIGNAL(currentIndexChanged(int)),
             this, SIGNAL(settingsChanged()));
-
+    connect(d->ui.locationCombo, SIGNAL(currentIndexChanged(int)),
+            this, SLOT(setSource(int)));
 }
 
 WeatherConfig::~WeatherConfig()
@@ -126,35 +159,67 @@ WeatherConfig::~WeatherConfig()
 
 void WeatherConfig::getNewStuff()
 {
-    KNS::Engine engine(this);
-    if (engine.init("plasmaweather.knsrc")) {
-        KNS::Entry::List entries = engine.downloadDialogModal(this);
-        if (entries.size() > 0) {
-            kDebug() << "About to ask WeatherEngine for plugin update!";
-            d->m_engine->setProperty("update", true);
-        }
-    }
+    KNS3::DownloadDialog dialog(QLatin1String( "kmediafactory.knsrc" ), this);
+    dialog.exec();
+    KNS3::Entry::List entries = dialog.changedEntries();
 }
 
 void WeatherConfig::setDataEngine(Plasma::DataEngine* dataengine)
 {
-    d->m_engine = dataengine;
-    d->searchDlg.setDataEngine(dataengine);
+    d->dataengine = dataengine;
+    //d->ions.clear();
+    qDeleteAll(d->validators);
+    d->validators.clear();
+    if (d->dataengine) {
+        const QVariantList plugins = d->dataengine->query(QLatin1String( "ions" )).values();
+        foreach (const QVariant& plugin, plugins) {
+            const QStringList pluginInfo = plugin.toString().split(QLatin1Char( '|' ));
+            if (pluginInfo.count() > 1) {
+                //kDebug() << "ion: " << pluginInfo[0] << pluginInfo[1];
+                //d->ions.insert(pluginInfo[1], pluginInfo[0]);
+                WeatherValidator *validator = new WeatherValidator(this);
+                connect(validator, SIGNAL(error(QString)), this, SLOT(validatorError(QString)));
+                connect(validator, SIGNAL(finished(QMap<QString,QString>)), this, SLOT(addSources(QMap<QString,QString>)));
+                validator->setDataEngine(dataengine);
+                validator->setIon(pluginInfo[1]);
+                d->validators.append(validator);
+            }
+        }
+    }
 }
 
-
-void WeatherConfig::setSource(const QString& source)
+void WeatherConfig::Private::validatorError(const QString &error)
 {
-    d->source = source;
-    QStringList list = source.split('|', QString::SkipEmptyParts);
-    if (list.count() > 0) {
-        d->ui.providerTextLabel->setText(d->searchDlg.nameForPlugin(list[0]));
+    kDebug() << error;
+}
+
+void WeatherConfig::Private::addSources(const QMap<QString, QString> &sources)
+{
+    QMapIterator<QString, QString> it(sources);
+
+    while (it.hasNext()) {
+        it.next();
+        QStringList list = it.value().split(QLatin1Char( '|' ), QString::SkipEmptyParts);
+        if (list.count() > 2) {
+            //kDebug() << list;
+            QString result = i18nc("A weather station location and the weather service it comes from",
+                                   "%1 (%2)", list[2], list[0]); // the names are too looong ions.value(list[0]));
+            ui.locationCombo->addItem(result, it.value());
+        }
     }
-    if (list.count() > 2) {
-        d->ui.cityTextLabel->setText(list[2]);
+
+    ++checkedInCount;
+    if (checkedInCount >= validators.count()) {
+        delete busyWidget;
+        busyWidget = 0;
+        kDebug() << ui.locationCombo->count();
+        if (ui.locationCombo->count() == 0 && ui.locationCombo->itemText(0).isEmpty()) {
+            const QString current = ui.locationCombo->currentText();
+            ui.locationCombo->addItem(i18n("No weather stations found for '%1'", current));
+            ui.locationCombo->lineEdit()->setText(current);
+        }
+        ui.locationCombo->showPopup();
     }
-    d->enableOK();
-    emit settingsChanged();
 }
 
 void WeatherConfig::setUpdateInterval(int interval)
@@ -199,32 +264,44 @@ void WeatherConfig::setVisibilityUnit(int unit)
     }
 }
 
-QString WeatherConfig::source()
+void WeatherConfig::setSource(const QString &source)
+{
+    //kDebug() << "source set to" << source;
+    const QStringList list = source.split(QLatin1Char( '|' ));
+    if (list.count() > 2) {
+        QString result = i18nc("A weather station location and the weather service it comes from",
+                               "%1 (%2)", list[2], list[0]);
+        d->ui.locationCombo->lineEdit()->setText(result);
+    }
+    d->source = source;
+}
+
+QString WeatherConfig::source() const
 {
     return d->source;
 }
 
-int WeatherConfig::updateInterval()
+int WeatherConfig::updateInterval() const
 {
     return d->ui.updateIntervalSpinBox->value();
 }
 
-int WeatherConfig::temperatureUnit()
+int WeatherConfig::temperatureUnit() const
 {
     return d->ui.temperatureComboBox->itemData(d->ui.temperatureComboBox->currentIndex()).toInt();
 }
 
-int WeatherConfig::pressureUnit()
+int WeatherConfig::pressureUnit() const
 {
     return d->ui.pressureComboBox->itemData(d->ui.pressureComboBox->currentIndex()).toInt();
 }
 
-int WeatherConfig::speedUnit()
+int WeatherConfig::speedUnit() const
 {
     return d->ui.speedComboBox->itemData(d->ui.speedComboBox->currentIndex()).toInt();
 }
 
-int WeatherConfig::visibilityUnit()
+int WeatherConfig::visibilityUnit() const
 {
     return d->ui.visibilityComboBox->itemData(d->ui.visibilityComboBox->currentIndex()).toInt();
 }
